@@ -157,6 +157,35 @@ channel_unfavourited_cb(GtFavouritesManager* mgr,
     PCLASS->check_empty(GT_CHANNELS_CONTAINER(self));
 }
 
+static void
+started_loading_favourites_cb(GtFavouritesManager* mgr,
+                              gpointer udata)
+{
+    GtChannelsContainerFavourite* self = GT_CHANNELS_CONTAINER_FAVOURITE(udata);
+    GtChannelsContainerFavouritePrivate* priv = gt_channels_container_favourite_get_instance_private(self);
+
+    PCLASS->show_load_spinner(GT_CHANNELS_CONTAINER(self), TRUE);
+}
+
+static void
+finished_loading_favourites_cb(GtFavouritesManager* mgr,
+                               gpointer udata)
+{
+    GtChannelsContainerFavourite* self = GT_CHANNELS_CONTAINER_FAVOURITE(udata);
+    GtChannelsContainerFavouritePrivate* priv = gt_channels_container_favourite_get_instance_private(self);
+
+    PCLASS->show_load_spinner(GT_CHANNELS_CONTAINER(self), FALSE);
+
+    PCLASS->clear_channels(GT_CHANNELS_CONTAINER(self));
+
+    PCLASS->append_channels(GT_CHANNELS_CONTAINER(self), main_app->fav_mgr->favourite_channels);
+
+    PCLASS->check_empty(GT_CHANNELS_CONTAINER(self));
+
+    for (GList* l = main_app->fav_mgr->favourite_channels; l != NULL; l = l->next)
+        g_signal_connect(G_OBJECT(l->data), "notify::updating", G_CALLBACK(channel_updating_cb), self);
+}
+
 static void
 finalize(GObject* object)
 {
@@ -224,18 +253,24 @@ gt_channels_container_favourite_init(GtChannelsContainerFavourite* self)
                                (GtkFlowBoxSortFunc) sort_favourites_by_name_and_online,
                                self, NULL);
 
+    //TODO: Change 'favourite' to 'follo' when logged in
+
     PCLASS->set_empty_info(GT_CHANNELS_CONTAINER(self),
                            "emblem-favorite-symbolic",
                            _("No channels favourited"),
                            _("Favourite channels that you like for them to show up here"));
 
-    PCLASS->append_channels(GT_CHANNELS_CONTAINER(self), main_app->fav_mgr->favourite_channels);
+    PCLASS->set_loading_info(GT_CHANNELS_CONTAINER(self), _("Loading favourites"));
 
-    PCLASS->check_empty(GT_CHANNELS_CONTAINER(self));
+    g_signal_connect(main_app->fav_mgr, "channel-favourited",
+                     G_CALLBACK(channel_favourited_cb), self);
+    g_signal_connect(main_app->fav_mgr, "channel-unfavourited",
+                     G_CALLBACK(channel_unfavourited_cb), self);
+    g_signal_connect(main_app->fav_mgr, "finished-loading-favourites",
+                     G_CALLBACK(finished_loading_favourites_cb), self);
+    g_signal_connect(main_app->fav_mgr, "started-loading-favourites",
+                     G_CALLBACK(started_loading_favourites_cb), self);
 
-    for (GList* l = main_app->fav_mgr->favourite_channels; l != NULL; l = l->next)
-        g_signal_connect(G_OBJECT(l->data), "notify::updating", G_CALLBACK(channel_updating_cb), self);
-
-    g_signal_connect(main_app->fav_mgr, "channel-favourited", G_CALLBACK(channel_favourited_cb), self);
-    g_signal_connect(main_app->fav_mgr, "channel-unfavourited", G_CALLBACK(channel_unfavourited_cb), self);
+    if (!gt_app_credentials_valid(main_app))
+        finished_loading_favourites_cb(main_app->fav_mgr, self);
 }
