@@ -19,11 +19,13 @@ struct _GtFavouritesManager
 };
 
 GtFavouritesManager* gt_favourites_manager_new(void);
-void gt_favourites_manager_load(GtFavouritesManager* self);
+void gt_favourites_manager_load_from_file(GtFavouritesManager* self);
+void gt_favourites_manager_load_from_twitch(GtFavouritesManager* self);
 void gt_favourites_manager_save(GtFavouritesManager* self);
 gboolean gt_favourites_manager_is_channel_favourited(GtFavouritesManager* self, GtChannel* chan);
 void gt_favourites_manager_attach_to_channel(GtFavouritesManager* self, GtChannel* chan);
 
+
 G_END_DECLS
 
 #endif
