@@ -1,31 +1,37 @@
 #include "gt-twitch.h"
 #include "gt-channel.h"
 #include "gt-game.h"
+#include "gt-app.h"
 #include "utils.h"
+#include "config.h"
 #include <libsoup/soup.h>
 #include <glib/gprintf.h>
+#include <glib/gi18n.h>
 #include <glib.h>
 #include <json-glib/json-glib.h>
 #include <string.h>
 #include <stdlib.h>
 
-#define ACCESS_TOKEN_URI    "http://api.twitch.tv/api/channels/%s/access_token"
-#define STREAM_PLAYLIST_URI "http://usher.twitch.tv/api/channel/hls/%s.m3u8?player=twitchweb&token=%s&sig=%s&allow_audio_only=true&allow_source=true&type=any&p=%d"
-#define TOP_CHANNELS_URI    "https://api.twitch.tv/kraken/streams?limit=%d&offset=%d&game=%s"
-#define TOP_GAMES_URI       "https://api.twitch.tv/kraken/games/top?limit=%d&offset=%d"
-#define SEARCH_CHANNELS_URI "https://api.twitch.tv/kraken/search/streams?q=%s&limit=%d&offset=%d"
-#define SEARCH_GAMES_URI    "https://api.twitch.tv/kraken/search/games?q=%s&type=suggest"
-#define STREAMS_URI         "https://api.twitch.tv/kraken/streams/%s"
-#define CHANNELS_URI        "https://api.twitch.tv/kraken/channels/%s"
-#define CHAT_BADGES_URI     "https://api.twitch.tv/kraken/chat/%s/badges/"
-#define TWITCH_EMOTE_URI    "http://static-cdn.jtvnw.net/emoticons/v1/%d/%d.0"
-#define CHANNEL_INFO_URI    "http://api.twitch.tv/api/channels/%s/panels"
-#define CHAT_SERVERS_URI    "https://api.twitch.tv/api/channels/%s/chat_properties"
+#define ACCESS_TOKEN_URI     "http://api.twitch.tv/api/channels/%s/access_token"
+#define STREAM_PLAYLIST_URI  "http://usher.twitch.tv/api/channel/hls/%s.m3u8?player=twitchweb&token=%s&sig=%s&allow_audio_only=true&allow_source=true&type=any&p=%d"
+#define TOP_CHANNELS_URI     "https://api.twitch.tv/kraken/streams?limit=%d&offset=%d&game=%s"
+#define TOP_GAMES_URI        "https://api.twitch.tv/kraken/games/top?limit=%d&offset=%d"
+#define SEARCH_CHANNELS_URI  "https://api.twitch.tv/kraken/search/streams?q=%s&limit=%d&offset=%d"
+#define SEARCH_GAMES_URI     "https://api.twitch.tv/kraken/search/games?q=%s&type=suggest"
+#define STREAMS_URI          "https://api.twitch.tv/kraken/streams/%s"
+#define CHANNELS_URI         "https://api.twitch.tv/kraken/channels/%s"
+#define CHAT_BADGES_URI      "https://api.twitch.tv/kraken/chat/%s/badges/"
+#define TWITCH_EMOTE_URI     "http://static-cdn.jtvnw.net/emoticons/v1/%d/%d.0"
+#define CHANNEL_INFO_URI     "http://api.twitch.tv/api/channels/%s/panels"
+#define CHAT_SERVERS_URI     "https://api.twitch.tv/api/channels/%s/chat_properties"
+#define FOLLOWS_URI          "https://api.twitch.tv/api/users/%s/follows/channels?limit=%d&offset=%d"
+#define FOLLOW_CHANNEL_URI   "https://api.twitch.tv/kraken/users/%s/follows/channels/%s?oauth_token=%s"
+#define UNFOLLOW_CHANNEL_URI "https://api.twitch.tv/kraken/users/%s/follows/channels/%s?oauth_token=%s"
+#define OAUTH_INFO_URI       "https://api.twitch.tv/kraken?oauth_token=%s"
 
 #define STREAM_INFO "#EXT-X-STREAM-INF"
 
-#define GT_TWITCH_ERROR_TOP_CHANNELS gt_spawn_twitch_top_channels_error_quark()
-#define GT_TWITCH_ERROR_SEARCH_CHANNELS gt_spawn_twitch_search_channels_error_quark()
+#define GT_TWITCH_ERROR gt_spawn_twitch_error_quark()
 
 typedef struct
 {
@@ -53,6 +59,10 @@ typedef struct
     gchar* str_1;
     gchar* str_2;
     gchar* str_3;
+
+    gboolean bool_1;
+    gboolean bool_2;
+    gboolean bool_3;
 } GenericTaskData;
 
 static GenericTaskData*
@@ -85,15 +95,9 @@ gt_twitch_stream_access_token_free(GtTwitchStreamAccessToken* token)
 }
 
 static GQuark
-gt_spawn_twitch_top_channels_error_quark()
+gt_spawn_twitch_error_quark()
 {
-    return g_quark_from_static_string("gt-twitch-top-channels-error");
-}
-
-static GQuark
-gt_spawn_twitch_search_channels_error_quark()
-{
-    return g_quark_from_static_string("gt-twitch-search-channels-error");
+    return g_quark_from_static_string("gt-twitch-error-quark");
 }
 
 GtTwitch*
@@ -169,6 +173,8 @@ send_message(GtTwitch* self, SoupMessage* msg)
 
     g_info("{GtTwitch} Sending message to uri '%s'", uri);
 
+    soup_message_headers_append(msg->request_headers, "Client-ID", CLIENT_ID);
+
     soup_session_send_message(priv->soup, msg);
 
     g_debug("{GtTwitch} Received code '%d' and response '%s'", msg->status_code, msg->response_body->data);
@@ -177,7 +183,7 @@ send_message(GtTwitch* self, SoupMessage* msg)
 
     g_free(uri);
 
-    return msg->status_code == SOUP_STATUS_OK;
+    return msg->status_code == SOUP_STATUS_OK || msg->status_code == SOUP_STATUS_NO_CONTENT;
 }
 
 static GDateTime*
@@ -274,7 +280,10 @@ parse_channel(GtTwitch* self, JsonReader* reader, GtChannelRawData* data)
     json_reader_end_member(reader);
 
     json_reader_read_member(reader, "status");
-    data->status = g_strdup(json_reader_get_string_value(reader));
+    if (json_reader_get_null_value(reader))
+        data->status = g_strdup(_("Untitled broadcast"));
+    else
+        data->status = g_strdup(json_reader_get_string_value(reader));
     json_reader_end_member(reader);
 
     json_reader_read_member(reader, "video_banner");
@@ -328,9 +337,10 @@ parse_game(GtTwitch* self, JsonReader* reader, GtGameRawData* data)
     json_reader_end_member(reader);
 
     json_reader_read_member(reader, "box");
-
     json_reader_read_member(reader, "large");
-    data->preview = gt_twitch_download_picture(self, json_reader_get_string_value(reader));
+
+    data->preview_url = g_strdup(json_reader_get_string_value(reader));
+
     json_reader_end_member(reader);
     json_reader_end_member(reader);
 }
@@ -344,9 +354,10 @@ gt_twitch_stream_access_token(GtTwitch* self, const gchar* channel)
     JsonNode* node;
     JsonReader* reader;
     GtTwitchStreamAccessToken* ret = NULL;
-    gchar uri[100];
+    gchar* uri = NULL;
+
+    uri = g_strdup_printf(ACCESS_TOKEN_URI, channel);
 
-    g_sprintf(uri, ACCESS_TOKEN_URI, channel);
     msg = soup_message_new("GET", uri);
 
     if (!send_message(self, msg))
@@ -374,6 +385,7 @@ gt_twitch_stream_access_token(GtTwitch* self, const gchar* channel)
     g_object_unref(reader);
     g_object_unref(parser);
 finish:
+    g_free(uri);
     g_object_unref(msg);
 
     return ret;
@@ -425,7 +437,7 @@ gt_twitch_all_streams(GtTwitch* self, const gchar* channel)
 {
     GtTwitchPrivate* priv = gt_twitch_get_instance_private(self);
     SoupMessage* msg;
-    gchar uri[500];
+    gchar* uri = NULL;
     GtTwitchStreamAccessToken* token;
     GList* ret = NULL;
 
@@ -433,7 +445,7 @@ gt_twitch_all_streams(GtTwitch* self, const gchar* channel)
 
     g_return_val_if_fail(token != NULL, NULL);
 
-    g_sprintf(uri, STREAM_PLAYLIST_URI, channel, token->token, token->sig, g_random_int_range(0, 999999));
+    uri = g_strdup_printf(STREAM_PLAYLIST_URI, channel, token->token, token->sig, g_random_int_range(0, 999999));
     msg = soup_message_new("GET", uri);
 
     if (!send_message(self, msg))
@@ -445,6 +457,7 @@ gt_twitch_all_streams(GtTwitch* self, const gchar* channel)
     ret = parse_playlist(msg->response_body->data);
 
 finish:
+    g_free(uri);
     g_object_unref(msg);
     gt_twitch_stream_access_token_free(token);
 
@@ -657,7 +670,7 @@ top_channels_async_cb(GTask* task,
 
     if (!ret)
     {
-        g_task_return_new_error(task, GT_TWITCH_ERROR_TOP_CHANNELS, GT_TWITCH_TOP_CHANNELS_ERROR_CODE,
+        g_task_return_new_error(task, GT_TWITCH_ERROR, GT_TWITCH_ERROR_TOP_CHANNELS,
                                 "Error getting top channels");
     }
     else
@@ -861,7 +874,7 @@ search_channels_async_cb(GTask* task,
 
     if (!ret)
     {
-        g_task_return_new_error(task, GT_TWITCH_ERROR_SEARCH_CHANNELS, GT_TWITCH_SEARCH_CHANNELS_ERROR_CODE,
+        g_task_return_new_error(task, GT_TWITCH_ERROR, GT_TWITCH_ERROR_SEARCH_CHANNELS,
                                 "Error searching channels");
     }
     else
@@ -966,7 +979,7 @@ gt_twitch_channel_raw_data(GtTwitch* self, const gchar* name)
 
     if (!send_message(self, msg))
     {
-        g_warning("{GtTwitch} Error sending message to get raw channel data");
+        g_warning("{GtTwitch} Error sending message to get raw channel data for channel '%s'", name);
         goto finish;
     }
 
@@ -1102,12 +1115,12 @@ void
 gt_twitch_game_raw_data_free(GtGameRawData* data)
 {
     g_free(data->name);
-    g_object_unref(data->preview);
+    g_free(data->preview_url);
     g_free(data);
 }
 
 GdkPixbuf*
-gt_twitch_download_picture(GtTwitch* self, const gchar* url)
+gt_twitch_download_picture(GtTwitch* self, const gchar* url, gint64 timestamp)
 {
     GtTwitchPrivate* priv = gt_twitch_get_instance_private(self);
 
@@ -1116,7 +1129,10 @@ gt_twitch_download_picture(GtTwitch* self, const gchar* url)
     if (!url || strlen(url) < 1)
         return NULL;
 
-    return utils_download_picture(priv->soup, url);
+    if (timestamp)
+        return utils_download_picture_if_newer(priv->soup, url, timestamp);
+    else
+        return utils_download_picture(priv->soup, url);
 }
 
 static void
@@ -1131,7 +1147,7 @@ download_picture_async_cb(GTask* task,
     if (g_task_return_error_if_cancelled(task))
         return;
 
-    ret = gt_twitch_download_picture(data->twitch, data->str_1);
+    ret = gt_twitch_download_picture(data->twitch, data->str_1, data->int_1);
 
     g_task_return_pointer(task, ret, (GDestroyNotify) g_object_unref);
 }
@@ -1139,6 +1155,7 @@ download_picture_async_cb(GTask* task,
 void
 gt_twitch_download_picture_async(GtTwitch* self,
                                  const gchar* url,
+                                 gint64 timestamp,
                                  GCancellable* cancel,
                                  GAsyncReadyCallback cb,
                                  gpointer udata)
@@ -1152,6 +1169,7 @@ gt_twitch_download_picture_async(GtTwitch* self,
     data = generic_task_data_new();
     data->twitch = self;
     data->str_1 = g_strdup(url);
+    data->int_1 = timestamp;
 
     g_task_set_task_data(task, data, (GDestroyNotify) generic_task_data_free);
 
@@ -1200,7 +1218,7 @@ gt_chat_badges(GtTwitch* self, const gchar* chan)
 {
     GtTwitchPrivate* priv = gt_twitch_get_instance_private(self);
     SoupMessage* msg;
-    gchar uri[100];
+    gchar* uri = NULL;
     JsonParser* parser;
     JsonNode* node;
     JsonReader* reader;
@@ -1208,7 +1226,7 @@ gt_chat_badges(GtTwitch* self, const gchar* chan)
 
     g_info("{GtTwitch} Getting chat badges for channel '%s'", chan);
 
-    g_sprintf(uri, CHAT_BADGES_URI, chan);
+    uri = g_strdup_printf(CHAT_BADGES_URI, chan);
 
     msg = soup_message_new("GET", uri);
 
@@ -1274,6 +1292,7 @@ gt_chat_badges(GtTwitch* self, const gchar* chan)
     g_object_unref(reader);
 
 finish:
+    g_free(uri);
     g_object_unref(msg);
 
     return ret;
@@ -1346,7 +1365,7 @@ gt_twitch_channel_info(GtTwitch* self, const gchar* chan)
 {
     GtTwitchPrivate* priv = gt_twitch_get_instance_private(self);
     SoupMessage* msg;
-    gchar uri[100];
+    gchar* uri = NULL;
     JsonParser* parser;
     JsonNode* node;
     JsonReader* reader;
@@ -1354,7 +1373,7 @@ gt_twitch_channel_info(GtTwitch* self, const gchar* chan)
 
     g_info("{GtTwitch} Getting channel info for '%s'", chan);
 
-    g_sprintf(uri, CHANNEL_INFO_URI, chan);
+    uri = g_strdup_printf(CHANNEL_INFO_URI, chan);
 
     msg = soup_message_new("GET", uri);
 
@@ -1408,7 +1427,7 @@ gt_twitch_channel_info(GtTwitch* self, const gchar* chan)
         json_reader_end_member(reader);
 
         json_reader_read_member(reader, "image");
-        panel->image = gt_twitch_download_picture(self, json_reader_get_string_value(reader));
+        panel->image = gt_twitch_download_picture(self, json_reader_get_string_value(reader), 0);
         json_reader_end_member(reader);
 
         if (json_reader_read_member(reader, "description"))
@@ -1430,6 +1449,7 @@ gt_twitch_channel_info(GtTwitch* self, const gchar* chan)
     g_object_unref(reader);
 
 finish:
+    g_free(uri);
     g_object_unref(msg);
 
     return ret;
@@ -1518,3 +1538,489 @@ finish:
 
     return ret;
 }
+
+GList*
+gt_twitch_follows(GtTwitch* self, const gchar* user_name,
+                  gint limit, gint offset)
+{
+    GtTwitchPrivate* priv = gt_twitch_get_instance_private(self);
+    SoupMessage* msg;
+    gchar* uri;
+    JsonParser* parser;
+    JsonNode* node;
+    JsonReader* reader;
+    GList* ret = NULL;
+
+    uri = g_strdup_printf(FOLLOWS_URI, user_name, limit, offset);
+    msg = soup_message_new(SOUP_METHOD_GET, uri);
+
+    if (!send_message(self, msg))
+    {
+        g_warning("{GtTwitch} Error sending message to get follows");
+        goto finish;
+    }
+
+    parser = json_parser_new();
+    json_parser_load_from_data(parser, msg->response_body->data, msg->response_body->length, NULL);
+    node = json_parser_get_root(parser);
+    reader = json_reader_new(node);
+
+    json_reader_read_member(reader, "follows");
+
+    gint max = json_reader_count_elements(reader);
+
+    for (gint i = 0; i < max; i++)
+    {
+        GtChannel* channel;
+        GtChannelRawData* raw = g_new0(GtChannelRawData, 1);;
+        gboolean live;
+
+        json_reader_read_element(reader, i);
+
+        json_reader_read_member(reader, "live");
+
+        if (json_reader_get_null_value(reader))
+        {
+            json_reader_end_member(reader);
+            json_reader_read_member(reader, "channel");
+            parse_channel(self, reader, raw);
+            raw->online = FALSE;
+        }
+        else
+        {
+            json_reader_end_member(reader);
+            json_reader_read_member(reader, "stream");
+            parse_stream(self, reader, raw);
+            raw->online = TRUE;
+        }
+
+        json_reader_end_member(reader);
+
+        channel = gt_channel_new(raw->name, raw->id);
+        g_object_force_floating(G_OBJECT(channel));
+        gt_channel_update_from_raw_data(channel, raw);
+
+        json_reader_end_element(reader);
+
+        ret = g_list_append(ret, channel);
+
+        gt_twitch_channel_raw_data_free(raw);
+    }
+
+    json_reader_end_member(reader);
+
+    g_object_unref(parser);
+    g_object_unref(reader);
+
+finish:
+    g_object_unref(msg);
+    g_free(uri);
+
+    return ret;
+}
+
+static void
+follows_async_cb(GTask* task,
+                 gpointer source,
+                 gpointer task_data,
+                 GCancellable* cancel)
+{
+    GenericTaskData* data = task_data;
+    GList* ret;
+
+    if (g_task_return_error_if_cancelled(task))
+        return;
+
+    ret = gt_twitch_follows(data->twitch, data->str_1, data->int_1, data->int_2);
+
+    if (!ret)
+        g_task_return_new_error(task, GT_TWITCH_ERROR, GT_TWITCH_ERROR_FOLLOWS,
+                                "Error getting follows");
+    else
+        g_task_return_pointer(task, ret, (GDestroyNotify) gt_channel_free_list);
+}
+
+void
+gt_twitch_follows_async(GtTwitch* self, const gchar* user_name,
+                        gint limit, gint offset,
+                        GCancellable* cancel,
+                        GAsyncReadyCallback cb,
+                        gpointer udata)
+{
+    GTask* task = NULL;
+    GenericTaskData* data = NULL;
+
+    task = g_task_new(NULL, cancel, cb, udata);
+    g_task_set_return_on_cancel(task, FALSE);
+
+    data = generic_task_data_new();
+    data->twitch = self;
+    data->int_1 = limit;
+    data->int_2 = offset;
+    data->str_1 = g_strdup(user_name);
+
+    g_task_set_task_data(task, data, (GDestroyNotify) generic_task_data_free);
+
+    g_task_run_in_thread(task, follows_async_cb);
+
+    g_object_unref(task);
+}
+
+GList*
+gt_twitch_follows_all(GtTwitch* self, const gchar* user_name)
+{
+    GtTwitchPrivate* priv = gt_twitch_get_instance_private(self);
+    SoupMessage* msg;
+    gchar* uri;
+    JsonParser* parser;
+    JsonNode* node;
+    JsonReader* reader;
+    GList* ret = NULL;
+    gint64 total;
+
+    uri = g_strdup_printf(FOLLOWS_URI, user_name, 99, 0);
+    msg = soup_message_new(SOUP_METHOD_GET, uri);
+
+    if (!send_message(self, msg))
+    {
+        g_warning("{GtTwitch} Error sending message to get follows");
+        goto finish;
+    }
+
+    parser = json_parser_new();
+    json_parser_load_from_data(parser, msg->response_body->data, msg->response_body->length, NULL);
+    node = json_parser_get_root(parser);
+    reader = json_reader_new(node);
+
+    json_reader_read_member(reader, "_total");
+    total = json_reader_get_int_value(reader);
+    json_reader_end_member(reader);
+
+    json_reader_read_member(reader, "follows");
+
+    gint max = json_reader_count_elements(reader);
+
+    for (gint i = 0; i < max; i++)
+    {
+        GtChannel* channel;
+        GtChannelRawData* raw = g_new0(GtChannelRawData, 1);;
+        gboolean live;
+
+        json_reader_read_element(reader, i);
+
+        json_reader_read_member(reader, "live");
+
+        if (json_reader_get_null_value(reader))
+        {
+            json_reader_end_member(reader);
+            json_reader_read_member(reader, "channel");
+            parse_channel(self, reader, raw);
+            raw->online = FALSE;
+        }
+        else
+        {
+            json_reader_end_member(reader);
+            json_reader_read_member(reader, "stream");
+            parse_stream(self, reader, raw);
+            raw->online = TRUE;
+        }
+
+        json_reader_end_member(reader);
+
+        channel = gt_channel_new(raw->name, raw->id);
+        gt_channel_update_from_raw_data(channel, raw);
+
+        json_reader_end_element(reader);
+
+        ret = g_list_append(ret, channel);
+
+        gt_twitch_channel_raw_data_free(raw);
+    }
+
+    if (total > 99)
+    {
+        for (gint j = 99; j < total; j += 99)
+            ret = g_list_concat(ret, gt_twitch_follows(self, user_name, 99, j));
+    }
+
+    json_reader_end_member(reader);
+
+    g_object_unref(parser);
+    g_object_unref(reader);
+
+finish:
+    g_object_unref(msg);
+    g_free(uri);
+
+    return ret;
+}
+
+static void
+follows_all_async_cb(GTask* task,
+                     gpointer source,
+                     gpointer task_data,
+                     GCancellable* cancel)
+{
+    GenericTaskData* data = task_data;
+    GList* ret;
+
+    if (g_task_return_error_if_cancelled(task))
+        return;
+
+    ret = gt_twitch_follows_all(data->twitch, data->str_1);
+
+    if (!ret)
+        g_task_return_new_error(task, GT_TWITCH_ERROR, GT_TWITCH_ERROR_FOLLOWS,
+                                "Error getting follows all");
+    else
+        g_task_return_pointer(task, ret, (GDestroyNotify) gt_channel_free_list);
+}
+
+void
+gt_twitch_follows_all_async(GtTwitch* self, const gchar* user_name,
+                            GCancellable* cancel,
+                            GAsyncReadyCallback cb,
+                            gpointer udata)
+{
+    GTask* task = NULL;
+    GenericTaskData* data = NULL;
+
+    task = g_task_new(NULL, cancel, cb, udata);
+    g_task_set_return_on_cancel(task, FALSE);
+
+    data = generic_task_data_new();
+    data->twitch = self;
+    data->str_1 = g_strdup(user_name);
+
+    g_task_set_task_data(task, data, (GDestroyNotify) generic_task_data_free);
+
+    g_task_run_in_thread(task, follows_all_async_cb);
+
+    g_object_unref(task);
+}
+
+gboolean
+gt_twitch_follow_channel(GtTwitch* self,
+                         const gchar* chan_name)
+{
+    GtTwitchPrivate* priv = gt_twitch_get_instance_private(self);
+    SoupMessage* msg;
+    gchar* uri;
+    gboolean ret = TRUE;
+
+    uri = g_strdup_printf(FOLLOW_CHANNEL_URI,
+                          gt_app_get_user_name(main_app),
+                          chan_name,
+                          gt_app_get_oauth_token(main_app));
+    msg = soup_message_new(SOUP_METHOD_PUT, uri);
+
+    if (!send_message(self, msg))
+    {
+        g_warning("{GtTwitch} Error sending message to follow channel '%s'", chan_name);
+
+        ret = FALSE;
+    }
+
+    g_free(uri);
+    g_object_unref(msg);
+
+    return ret;
+}
+
+static void
+follow_channel_async_cb(GTask* task,
+                        gpointer source,
+                        gpointer task_data,
+                        GCancellable* cancel)
+{
+    GenericTaskData* data = task_data;
+    gboolean ret;
+
+    ret = gt_twitch_follow_channel(data->twitch, data->str_1);
+
+    if (!ret)
+        g_task_return_new_error(task, GT_TWITCH_ERROR, GT_TWITCH_ERROR_FOLLOW_CHANNEL,
+                                "Error following channel '%s'", data->str_1);
+    else
+        g_task_return_boolean(task, ret);
+}
+
+// Not cancellable; hard to guarantee that channel is not followed
+void
+gt_twitch_follow_channel_async(GtTwitch* self,
+                               const gchar* chan_name,
+                               GAsyncReadyCallback cb,
+                               gpointer udata)
+{
+    GTask* task = NULL;
+    GenericTaskData* data = NULL;
+
+    task = g_task_new(NULL, NULL, cb, udata);
+
+    data = generic_task_data_new();
+    data->twitch = self;
+    data->str_1 = g_strdup(chan_name);
+
+    g_task_set_task_data(task, data, (GDestroyNotify) generic_task_data_free);
+
+    g_task_run_in_thread(task, follow_channel_async_cb);
+
+    g_object_unref(task);
+}
+
+gboolean
+gt_twitch_unfollow_channel(GtTwitch* self,
+                           const gchar* chan_name)
+{
+    GtTwitchPrivate* priv = gt_twitch_get_instance_private(self);
+    SoupMessage* msg;
+    gchar* uri;
+    gboolean ret = TRUE;
+
+    uri = g_strdup_printf(UNFOLLOW_CHANNEL_URI,
+                          gt_app_get_user_name(main_app),
+                          chan_name,
+                          gt_app_get_oauth_token(main_app));
+    msg = soup_message_new(SOUP_METHOD_DELETE, uri);
+
+    if (!send_message(self, msg))
+    {
+        g_warning("{GtTwitch} Error sending message to unfollow channel '%s'", chan_name);
+
+        ret = FALSE;
+    }
+
+    g_free(uri);
+    g_object_unref(msg);
+
+    return ret;
+}
+static void
+unfollow_channel_async_cb(GTask* task,
+                          gpointer source,
+                          gpointer task_data,
+                          GCancellable* cancel)
+{
+    GenericTaskData* data = task_data;
+    gboolean ret;
+
+    ret = gt_twitch_unfollow_channel(data->twitch, data->str_1);
+
+    if (!ret)
+        g_task_return_new_error(task, GT_TWITCH_ERROR, GT_TWITCH_ERROR_UNFOLLOW_CHANNEL,
+                                "Error unfollowing channel '%s'", data->str_1);
+    else
+        g_task_return_boolean(task, ret);
+}
+
+// Not cancellable; hard to guarantee that channel is not unfollowed
+void
+gt_twitch_unfollow_channel_async(GtTwitch* self,
+                               const gchar* chan_name,
+                               GAsyncReadyCallback cb,
+                               gpointer udata)
+{
+    GTask* task = NULL;
+    GenericTaskData* data = NULL;
+
+    task = g_task_new(NULL, NULL, cb, udata);
+
+    data = generic_task_data_new();
+    data->twitch = self;
+    data->str_1 = g_strdup(chan_name);
+
+    g_task_set_task_data(task, data, (GDestroyNotify) generic_task_data_free);
+
+    g_task_run_in_thread(task, unfollow_channel_async_cb);
+
+    g_object_unref(task);
+}
+
+gchar*
+gt_twitch_user_name(GtTwitch* self,
+                    GError** error)
+{
+    GtTwitchPrivate* priv = gt_twitch_get_instance_private(self);
+    SoupMessage* msg;
+    gchar* uri;
+    gchar* ret = NULL;
+    JsonParser* parser;
+    JsonNode* node;
+    JsonReader* reader;
+
+    uri = g_strdup_printf(OAUTH_INFO_URI,
+                          gt_app_get_oauth_token(main_app));
+    msg = soup_message_new(SOUP_METHOD_GET, uri);
+
+    if (!send_message(self, msg))
+    {
+        //TODO: Use new logging function
+        g_warning("{GtTwitch} Unable to get username");
+
+        g_set_error(error, GT_TWITCH_ERROR, GT_TWITCH_ERROR_USER_NAME,
+                    "Unable to get username");
+
+        goto finish;
+    }
+
+    parser = json_parser_new();
+    json_parser_load_from_data(parser, msg->response_body->data, msg->response_body->length, NULL);
+    node = json_parser_get_root(parser);
+    reader = json_reader_new(node);
+
+    json_reader_read_member(reader, "token");
+
+    json_reader_read_member(reader, "user_name");
+    ret = g_strdup(json_reader_get_string_value(reader));
+    json_reader_end_member(reader);
+
+    json_reader_end_member(reader);
+
+    g_object_unref(parser);
+    g_object_unref(reader);
+
+finish:
+    g_free(uri);
+    g_object_unref(msg);
+
+    return ret;
+
+}
+
+static void
+user_name_async_cb(GTask* task,
+                  gpointer source,
+                  gpointer task_data,
+                  GCancellable* cancel)
+{
+    GenericTaskData* data = task_data;
+    gchar* ret = NULL;
+    GError* error = NULL;
+
+    ret = gt_twitch_user_name(data->twitch, &error);
+
+    if (!ret)
+        g_task_return_error(task, error);
+    else
+        g_task_return_pointer(task, ret, g_free);
+}
+
+void
+gt_twitch_user_name_async(GtTwitch* self,
+                          GAsyncReadyCallback cb,
+                          gpointer udata)
+{
+    GTask* task = NULL;
+    GenericTaskData* data = NULL;
+
+    task = g_task_new(NULL, NULL, cb, udata);
+
+    data = generic_task_data_new();
+    data->twitch = self;
+
+    g_task_set_task_data(task, data, (GDestroyNotify) generic_task_data_free);
+
+    g_task_run_in_thread(task, user_name_async_cb);
+
+    g_object_unref(task);
+}
