@@ -1,4 +1,7 @@
 #include "utils.h"
+#include "config.h"
+#include <glib/gstdio.h>
+#include <glib.h>
 #include <stdlib.h>
 #include <string.h>
 
@@ -30,6 +33,33 @@ utils_container_clear(GtkContainer* cont)
     }
 }
 
+gint64
+utils_timestamp_file(const gchar* filename)
+{
+    int ret;
+    GStatBuf file_stat;
+
+    ret = g_stat(filename, &file_stat);
+
+    if (ret)
+        return 0;
+
+    return file_stat.st_mtim.tv_sec;
+}
+
+gint64
+utils_timestamp_now(void)
+{
+    gint64 timestamp;
+    GDateTime* now;
+
+    now = g_date_time_new_now_utc();
+    timestamp = g_date_time_to_unix(now);
+    g_date_time_unref(now);
+
+    return timestamp;
+}
+
 void
 utils_pixbuf_scale_simple(GdkPixbuf** pixbuf, gint width, gint height, GdkInterpType interp)
 {
@@ -41,6 +71,19 @@ utils_pixbuf_scale_simple(GdkPixbuf** pixbuf, gint width, gint height, GdkInterp
     *pixbuf = tmp;
 }
 
+static gint64
+utils_http_full_date_to_timestamp(const char* string)
+{
+    gint64 ret;
+    SoupDate* tmp;
+
+    tmp = soup_date_new_from_string(string);
+    ret = soup_date_to_time_t(tmp);
+    soup_date_free(tmp);
+
+    return ret;
+}
+
 GdkPixbuf*
 utils_download_picture(SoupSession* soup, const gchar* url)
 {
@@ -50,6 +93,7 @@ utils_download_picture(SoupSession* soup, const gchar* url)
     GError* err = NULL;
 
     msg = soup_message_new("GET", url);
+    soup_message_headers_append(msg->request_headers, "Client-ID", CLIENT_ID);
     input = soup_session_send(soup, msg, NULL, &err);
 
     if (err)
@@ -69,6 +113,33 @@ utils_download_picture(SoupSession* soup, const gchar* url)
     return ret;
 }
 
+GdkPixbuf*
+utils_download_picture_if_newer(SoupSession* soup, const gchar* url, gint64 timestamp)
+{
+    SoupMessage* msg;
+    guint soup_status;
+    const gchar* last_modified;
+    GdkPixbuf* ret;
+
+    msg = soup_message_new(SOUP_METHOD_HEAD, url);
+    soup_message_headers_append(msg->request_headers, "Client-ID", CLIENT_ID);
+    soup_status = soup_session_send_message(soup, msg);
+
+    if (SOUP_STATUS_IS_SUCCESSFUL(soup_status) &&
+        (last_modified = soup_message_headers_get_one(msg->response_headers, "Last-Modified")) != NULL &&
+        utils_http_full_date_to_timestamp(last_modified) < timestamp)
+    {
+        g_info("{Utils} No new content at url '%s'", url);
+        ret = NULL;
+    }
+    else
+        ret = utils_download_picture(soup, url);
+
+    g_object_unref(msg);
+
+    return ret;
+}
+
 const gchar*
 utils_search_key_value_strv(gchar** strv, const gchar* key)
 {
@@ -155,3 +226,9 @@ utils_connect_link(GtkWidget* widget, const gchar* link)
     utils_connect_mouse_hover(widget);
     g_signal_connect(widget, "button-press-event", G_CALLBACK(utils_mouse_clicked_link_cb), tmp);
 }
+
+gboolean
+utils_str_empty(const gchar* str)
+{
+    return !(str && strlen(str) > 0);
+}
